# -*- coding: utf-8 -*-
""" Main NDL Search client.
"""

from abc import abstractmethod
from abc import ABCMeta

from .api.sru import SRUApi


class AbstractClient(metaclass=ABCMeta):
    @abstractmethod
    def __init__(self, query):
        self.api = None
        self.query = query

    @abstractmethod
    def get_response(self):
        return self.query.get()


class SRUClient(AbstractClient):
    def __init__(self, query):
        self.api = SRUApi(query.payload())

    def get_response(self):
        return self.api.get()

    def get_srresponse(self):
        return self.api.parse()

    def set_start_record(self, startRecord):
        self.api.startRecord = startRecord

    def set_maximum_records(self, maximumRecords):
        self.api.maximumRecords = maximumRecords

    def set_record_packing(self, recordPacking):
        self.api.recordPacking = recordPacking

    def set_record_schema(self, recordSchema):
        self.api.recordSchema = recordSchema

    def set_inprocess(self, inprocess):
        self.api.inprocess = inprocess

    def set_only_bib(self, onlyBib):
        self.api.onlyBib = onlyBib



# -*- coding: utf-8 -*-


class CQL(object):
    """ 検索クエリ. """
    query = ''

    """ データプロバイダID. """
    dpid = ''

    """ データプロバイダグループID. """
    dpgroupid = ''

    """ タイトル. """
    title = ''

    """ 作成者. """
    creator = ''

    """ 出版社. """
    publisher = ''

    """ 分類(NDC,NDLC,LCC,DCC,UDC) """
    ndc = ''

    """ 分類(NDLC) """
    ndlc = ''

    """ 内容記述. """
    description = ''

    """ 主題. """
    subject = ''

    """ ISBN. """
    isbn = ''

    """ ISSN. """
    issn = ''

    """ 全国書誌番号. """
    jpno = ''

    """ 開始出版年月 (YYYY-MM-DDD). """
    fromdate = ''

    """ 終了出版年月日 (YYY-MM-DDD). """
    untildate = ''

    """ 検索対象項目は国立国会図書館サーチの簡易検索と同一. """
    anywhere = ''

    """ 国立国会図書館サーチ内部での書誌のアイテム番号. """
    itemno = ''

    """ 資料種別.
    国立国会図書館サーチの詳細検索の資料種別に対応
    '1': 本
    '2': 記事
    '3': 新聞
    '4': 児童書
    '5': レファレンス情報
    '6': デジタル資料
    '7': その他
    '8': 障害者向け資料
    '9': 立法情報
    """
    mediatype = ''

    """ ソートの基準となる参照名を記述する.
    [title|creator|created_at|modified_date|issued_date]
    /[sort.ascending|sort.descending]
    """
    sortBy = ''

    def __init__(self, query=''):
        self.query = query

    def is_empty(self, param):
        return True if param is '' else False

    def is_empty_query(self):
        return True if self.query is '' else False

    def commit(self):
        if not self.is_empty(self.dpid):
            self.query += 'dpid%3d"{}"'.format(self.dpid)

        if not self.is_empty(self.dpgroupid):
            if self.is_empty_query():
                self.query += 'dpgroupid%3d"{}"'.format(self.dpgroupid)
            else:
                self.query += ' AND dpgroupid="{}"'.format(self.dpgroupid)

        if not self.is_empty(self.title):
            if self.is_empty_query():
                self.query += 'title%3d"{}"'.format(self.title)
            else:
                self.query += ' AND title="{}"'.format(self.title)

        if not self.is_empty(self.creator):
            if self.is_empty_query():
                self.query += 'creator%3d"{}"'.format(self.creator)
            else:
                self.query += ' AND creator="{}"'.format(self.creator)

        if not self.is_empty(self.publisher):
            if self.is_empty_query():
                self.query += 'publisher%3d"{}"'.format(self.publisher)
            else:
                self.query += ' AND publisher="{}"'.format(self.publisher)

        if not self.is_empty(self.ndc):
            if self.is_empty_query():
                self.query += 'ndc%3d"{}"'.format(self.ndc)
            else:
                self.query += ' AND ndc="{}"'.format(self.ndc)

        if not self.is_empty(self.ndlc):
            if self.is_empty_query():
                self.query += 'ndlc%3d"{}"'.format(self.ndlc)
            else:
                self.query += ' AND ndlc="{}"'.format(self.ndlc)

        if not self.is_empty(self.description):
            if self.is_empty_query():
                self.query += 'description%3d"{}"'.format(self.description)
            else:
                self.query += ' AND description="{}"'.format(self.description)

        if not self.is_empty(self.subject):
            if self.is_empty_query():
                self.query += 'subject%3d"{}"'.format(self.subject)
            else:
                self.query += ' AND subject="{}"'.format(self.subject)

        if not self.is_empty(self.isbn):
            if self.is_empty_query():
                self.query += 'isbn%3d"{}"'.format(self.isbn)
            else:
                self.query += ' AND isbn="{}"'.format(self.isbn)

        if not self.is_empty(self.issn):
            if self.is_empty_query():
                self.query += 'issn%3d"{}"'.format(self.issn)
            else:
                self.query += ' AND issn="{}"'.format(self.issn)

        if not self.is_empty(self.jpno):
            if self.is_empty_query():
                self.query += 'jpno%3d"{}"'.format(self.jpno)
            else:
                self.jpno += ' AND jpno="{}"'.format(self.jpno)

        if not self.is_empty(self.fromdate):
            if self.is_empty_query():
                self.query += 'from%3d"{}"'.format(self.fromdate)
            else:
                self.query += ' AND from="{}"'.format(self.fromdate)

        if not self.is_empty(self.untildate):
            if self.is_empty_query():
                self.query += 'until%3d"{}"'.format(self.untildate)
            else:
                self.query += ' AND until="{}"'.format(self.untildate)

        if not self.is_empty(self.itemno):
            if self.is_empty_query():
                self.query += 'itemno%3d"{}"'.format(self.itemno)
            else:
                self.query += ' AND itemno="{}"'.format(self.itemno)

        if not self.is_empty(self.mediatype):
            if self.is_empty_query():
                self.query += 'mediatype%3d"{}"'.format(self.mediatype)
            else:
                self.query += ' AND mediatype="{}"'.format(self.mediatype)

        if not self.is_empty(self.sortBy):
            if self.is_empty_query():
                self.query += 'sortBy%3d"{}"'.format(self.sortBy)
            else:
                self.query += ' AND sortBy="{}"'.format(self.sortBy)

    def payload(self):
        self.commit()
        return self.query



# -*- coding: utf-8 -*-
""" SRU: Search/Retrieval via URL APIクラス."""

import requests
from xml.etree import ElementTree

from .abs_api import AbstractAPI


class RecordData(object):
    title = ''
    creator = ''
    descriptions = {}
    publisher = ''
    language = ''

    def __init__(self):
        pass

class Record(object):
    recordSchema = ''
    recordPacking = ''
    recordData = None
    recordPosition = 0

    def __init__(self):
        pass


class extraResponseData(object):
    facets = {}

    def __init__(self):
        pass


class searchRetrieveResponse(object):
    version = ''
    numberOfRecords = 0
    nextRecordPosition = ''
    extraResponseData = None
    records = []

    def __init__(self):
        pass


class SRUApi(AbstractAPI):
    URL = 'http://iss.ndl.go.jp/api/sru'

    def __init__(self, query):
        self.operation = 'searchRetrieve'
        self.version = '1.2'

        """ 検索条件(CQL). """
        self.query = query

        """ 開始位置. """
        self.startRecord = '1'

        """ 最大取得件数. """
        self.maximumRecords = '200'

        """ xml or string. """
        self.recordPacking = 'string'

        """ 取得データのスキーマ. """
        self.recordSchema = 'dc'

        """ NDL新着書誌情報のみを取得. """
        self.inprocess = 'false'

        """ 書誌情報のみを取得. """
        self.onlyBib = 'false'

    def make_query(self):
        self.query = 'operation={}&query={}'.format(
            self.operation,
            self.query,
        )

        if self.startRecord != '1':
            self.query += '&startRecord={}'.format(self.startRecord)

        if self.maximumRecords != '200':
            self.query += '&maximumRecords={}'.format(self.maximumRecords)

        if self.recordPacking != 'string':
            self.query += '&recordPacking={}'.format(self.recordPacking)

        if self.recordSchema != 'dc':
            self.query += '&recordSchema={}'.format(self.recordSchema)

        if self.inprocess != 'false':
            self.query += '&inprocess={}'.format(self.inprocess)

        if self.onlyBib != 'false':
            self.query += '&onlyBib={}'.format(self.onlyBib)

    def get(self):
        self.make_query()
        res = requests.get(self.URL, params=self.query)

        return res

    def parse(self):
        res = searchRetrieveResponse()
        extra_resdata = extraResponseData()

        root = ElementTree.fromstring(self.get().text)

        res.version = root.find('{http://www.loc.gov/zing/srw/}version').text
        res.numberOfRecords = root.find('{http://www.loc.gov/zing/srw/}numberOfRecords').text
        res.nextRecordPosition = root.find('{http://www.loc.gov/zing/srw/}numberOfRecords').text

        # extraResponseData
        ext_root = ElementTree.fromstring(root.find('{http://www.loc.gov/zing/srw/}extraResponseData').text)
        for child in ext_root:

            name = child.get('name')
            if name == 'REPOSITORY_NO':
                repository_no = {}
                for child_repo in child:
                    repository_no[child_repo.get('name')] = child_repo.text

                extra_resdata.facets['REPOSITORY_NO'] = repository_no

            elif name == 'NDC':
                ndc = {}
                for child_ndc in child:
                    ndc[child_ndc.get('name')] = child_ndc.text

                extra_resdata.facets['NDC'] = ndc

            elif name == 'ISSUED_DATE':
                issued_date = {}
                for child_issued in child:
                    issued_date[child_issued.get('name')] = child_issued.text

                extra_resdata.facets['ISSUED_DATE'] = issued_date

            elif name == 'LIBRARY':
                library = {}
                for child_lib in child:
                    library[child_lib.get('name')] = child_lib.text

                extra_resdata.facets['LIBRARY'] = library

        # records
        records = []
        records_root = root.find('{http://www.loc.gov/zing/srw/}records')
        for record in records_root:
            tmp_record = Record()
            tmp_record.recordSchema= record.find('{http://www.loc.gov/zing/srw/}recordSchema').text
            tmp_record.recordPacking = record.find('{http://www.loc.gov/zing/srw/}recordPacking').text
            tmp_record.recordPosition = record.find('{http://www.loc.gov/zing/srw/}recordPosition').text

            tmp_record_data = RecordData()
            record_data_root = ElementTree.fromstring(record.find('{http://www.loc.gov/zing/srw/}recordData').text)

            title = record_data_root.find('{http://purl.org/dc/elements/1.1/}title')
            if title is not None:
                tmp_record_data.title = title.text

            creator = record_data_root.find('{http://purl.org/dc/elements/1.1/}creator')
            if creator is not None:
                tmp_record_data.creator = creator.text

            publisher = record_data_root.find('{http://purl.org/dc/elements/1.1/}publisher')
            if publisher is not None:
                tmp_record_data.publisher = publisher.text

            language = record_data_root.find('{http://purl.org/dc/elements/1.1/}language')
            if language is not None:
                tmp_record_data.language = language.text

            subject = record_data_root.find('{http://purl.org/dc/elements/1.1/}subject')
            if subject is not None:
                tmp_record_data.subject = subject.text

            descriptions = record_data_root.find('{http://purl.org/dc/elements/1.1/}description')
            if descriptions is not None:
                tmp_record_data.descriptions = descriptions

            tmp_record.recordData = tmp_record_data
            records.append(tmp_record)

        res.extraResponseData = extra_resdata
        res.records = records

        return res



import os

from distutils.core import setup

def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == "":
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

package_dir = "pyndlsearch"

packages = []
for dirpath, dirnames, filenames in os.walk(package_dir):
    for i, dirname in enumerate(dirnames):
        if dirname.startswith("."):
            del dirnames[i]
    if "__init__.py" in filenames:
        packages.append(".".join(fullsplit(dirpath)))

setup(
    name = 'pyndlsearch',
    version = '1.0',
    description = 'A Python Wrapper for NDL Search API.',
    keywords = 'api, ndl, search, wrapper',
    license = 'MIT License',
    author = 'nocotan',
    author_email = 'noconoco.lib@gmail.com',
    url = 'http://github.com/nocotan/pyndlsearch/',
    packages = packages,
)
